@@ -14,8 +14,8 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; see the file COPYING.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301, USA.
 */
 
 #include "arkplugin.h"
@@ -31,6 +31,7 @@
 #include <kpopupmenu.h>
 #include <kgenericfactory.h>
 #include <kurl.h>
+#include <kio/netaccess.h>
 
 #include <qdir.h>
 #include <qcstring.h>
@@ -57,9 +58,15 @@ ArkMenu::ArkMenu( KonqPopupMenu * popupmenu, const char *name, const QStringList
     KGlobal::locale()->insertCatalogue("ark_plugin");
 
     extMimeTypes();
-    m_list = popupmenu->fileItemList();
-    unsigned int itemCount = m_list.count();
-    KFileItemListIterator it( m_list );
+    KFileItemList itemList = popupmenu->fileItemList();
+    for ( KFileItem *item = itemList.first(); item; item = itemList.next() )
+    {
+        m_urlList.append( item->url() );
+        m_urlStringList.append( item->url().url() );
+    }
+    m_dir = popupmenu->url().url() + "/";
+    unsigned int itemCount = m_urlList.count();
+    KFileItemListIterator it( itemList );
     KFileItem * item;
     bool hasArchives = false;
     bool hasOther = false;
@@ -82,15 +89,15 @@ ArkMenu::ArkMenu( KonqPopupMenu * popupmenu, const char *name, const QStringList
     QString ext;
     KActionMenu * actionMenu;
     KAction * action;
-    if ( hasOther )
+    if ( hasOther &&  itemList.first()->name()!="." && popupmenu->protocolInfo().supportsWriting() )	// don't try to compress if we right click on a folder without files selected
     {
         compMimeTypes();
         actionMenu = new KActionMenu( i18n( "Compress" ), "ark", actionCollection(), "ark_compress_menu"  );
         m_ext = m_conf->readEntry( "LastExtension", ".tar.gz" );
         if ( itemCount == 1 )
         {
-            item = m_list.first();
-            m_name = m_list.first()->name();
+            item = itemList.first();
+            m_name = itemList.first()->name();
             action = new KAction( i18n( "Compress as %1" ).arg( m_name + m_ext ), 0, this,
                                     SLOT( slotCompressAsDefault() ), actionCollection() );
         }
@@ -107,7 +114,7 @@ ArkMenu::ArkMenu( KonqPopupMenu * popupmenu, const char *name, const QStringList
 
 
         m_addToMenu = new KActionMenu( i18n( "Add To" ), actionCollection(), "arkaddtomnu" );
-        if ( m_list.first()->url().isLocalFile() )
+        if ( itemList.first()->url().isLocalFile() )
             actionMenu->insert( m_addToMenu );
 
         connect( m_compAsMenu->popupMenu(), SIGNAL( aboutToShow() ),
@@ -124,29 +131,37 @@ ArkMenu::ArkMenu( KonqPopupMenu * popupmenu, const char *name, const QStringList
 
     if ( !hasOther && hasArchives )
     {
-        actionMenu = new KActionMenu( i18n( "Extract" ), "ark", actionCollection(), "ark_extract_menu"  );
-
-        action = new KAction( i18n( "Extract Here" ), 0, this,
-                                SLOT( slotExtractHere() ), actionCollection() );
-        actionMenu->insert( action );
-        // stolen from arkwidget.cpp
-        if ( itemCount == 1 )
+        if ( popupmenu->protocolInfo().supportsWriting() )
         {
-            QString targetName = m_list.first()->name();
-            stripExtension( targetName );
-            action = new KAction( i18n( "Extract to %1" ).arg( targetName ), 0, this,
-                                    SLOT( slotExtractToSubfolders() ), actionCollection() );
+            actionMenu = new KActionMenu( i18n( "Extract" ), "ark", actionCollection(), "ark_extract_menu"  );
+
+            action = new KAction( i18n( "Extract Here" ), 0, this,
+                                    SLOT( slotExtractHere() ), actionCollection() );
+            actionMenu->insert( action );
+            // stolen from arkwidget.cpp
+            if ( itemCount == 1 )
+            {
+                QString targetName = itemList.first()->name();
+                stripExtension( targetName );
+                action = new KAction( i18n( "Extract to %1" ).arg( targetName ), 0, this,
+                                        SLOT( slotExtractToSubfolders() ), actionCollection() );
+            }
+            else
+            {
+                action = new KAction( i18n( "Extract to Subfolders" ), 0, this,
+                                        SLOT( slotExtractToSubfolders() ), actionCollection() );
+            }
+            actionMenu->insert( action );
+            action = new KAction( i18n( "Extract To..." ), 0 , this,
+                                    SLOT( slotExtractTo() ), actionCollection() );
+            actionMenu->insert( action );
+            addAction( actionMenu );
         }
         else
         {
-            action = new KAction( i18n( "Extract to Subfolders" ), 0, this,
-                                    SLOT( slotExtractToSubfolders() ), actionCollection() );
+           action = new KAction( i18n( "Extract To..." ), "ark", 0,  this, SLOT( slotExtractTo() ), actionCollection(), "ark_extract_menu" );
+           addAction( action );
         }
-        actionMenu->insert( action );
-        action = new KAction( i18n( "Extract To..." ), 0 , this,
-                                SLOT( slotExtractTo() ), actionCollection() );
-        actionMenu->insert( action );
-        addAction( actionMenu );
     }
     addSeparator();
 }
@@ -177,7 +192,7 @@ void ArkMenu::slotPrepareCompAsMenu()
         (*eit).remove( '*' );
         if ( *eit == ".tar.bz" )                   // tbz mimetype, has tar.bz as first entry :}
             *eit = ".tar.bz2";
-        if ( m_list.count() == 1 )
+        if ( m_urlList.count() == 1 )
         {
             action = new KAction( m_name + (*eit), 0, m_compAsMapper,
                     SLOT( map() ), actionCollection() );
@@ -221,7 +236,7 @@ void ArkMenu::slotPrepareAddToMenu()
     QCString actionName;
     QStringList::Iterator mit;
     KURL archive;
-    QDir dir( m_list.first()->url().directory() );
+    QDir dir( m_urlList.first().directory() );
     QStringList entries = dir.entryList();
     QStringList::Iterator uit = entries.begin();
     for ( ; uit != entries.end(); ++uit )
@@ -244,18 +259,22 @@ void ArkMenu::slotPrepareAddToMenu()
 
 void ArkMenu::compMimeTypes()
 {
+    unsigned int itemCount = m_urlList.count();
+
     bool havegz = false;
     if ( !KStandardDirs::findExe( "gzip" ).isNull() && m_conf->readBoolEntry( "UseGz", true ) )
     {
         havegz = true;
-        m_archiveMimeTypes << "application/x-gzip";
+        //.gz can only compress one file, not multiple 
+        if ( itemCount == 1 ) m_archiveMimeTypes << "application/x-gzip";
     }
 
     bool havebz2 = false;
     if ( !KStandardDirs::findExe( "bzip2" ).isNull() && m_conf->readBoolEntry( "UseBzip2", true ) )
     {
         havebz2 = true;
-        m_archiveMimeTypes << "application/x-bzip2";
+        //.bz2 can only compress one file, not multiple
+        if ( itemCount == 1 ) m_archiveMimeTypes << "application/x-bzip2";
     }
 
     bool havelzop = false;
@@ -271,7 +290,10 @@ void ArkMenu::compMimeTypes()
         if ( havegz )
             m_archiveMimeTypes << "application/x-tgz";
         if ( havebz2 )
+        {
             m_archiveMimeTypes << "application/x-tbz";
+            m_archiveMimeTypes << "application/x-tbz2";
+        }
         if ( havelzop )
             m_archiveMimeTypes << "application/x-tzo";
     }
@@ -288,7 +310,7 @@ void ArkMenu::compMimeTypes()
     }
 
     if ( !KStandardDirs::findExe( "rar" ).isNull() && m_conf->readBoolEntry( "UseRar", true ) )
-        m_archiveMimeTypes << "application/x-rar";
+        m_archiveMimeTypes << "application/x-rar" << "application/x-rar-compressed";
 
     if ( !KStandardDirs::findExe( "zoo" ).isNull() && m_conf->readBoolEntry( "UseZoo", false ) )
         m_archiveMimeTypes << "application/x-zoo";
@@ -333,7 +355,10 @@ void ArkMenu::extMimeTypes()
         if ( havegz )
             m_extractMimeTypes << "application/x-tgz";
         if ( havebz2 )
+        {
             m_extractMimeTypes << "application/x-tbz";
+            m_extractMimeTypes << "application/x-tbz2";
+        }
         if ( havelzop )
             m_extractMimeTypes << "application/x-tzo";
     }
@@ -345,7 +370,7 @@ void ArkMenu::extMimeTypes()
         m_extractMimeTypes << "application/x-zip" << "application/x-jar";
 
     if ( !KStandardDirs::findExe( "unrar" ).isNull() )
-        m_extractMimeTypes << "application/x-rar";
+        m_extractMimeTypes << "application/x-rar" << "application/x-rar-compressed";
 
     if ( !KStandardDirs::findExe( "zoo" ).isNull() )
         m_extractMimeTypes << "application/x-zoo";
@@ -381,15 +406,28 @@ void ArkMenu::slotCompressAs( int pos )
     QCString name;
     QString extension, mimeType;
     KURL target;
-    KFileItemListIterator it( m_list );
-    KFileItem * item;
-    while (  ( item = it.current() ) != 0 )
+    QStringList filelist( m_urlStringList );
+
+    //if KMimeType returns .ZIP or .RAR. convert them to lowercase
+    if ( m_extensionList[ pos ].contains ( ".ZIP" ) )
+         m_extensionList[ pos ] = ".zip";
+
+    if ( m_extensionList[ pos ].contains ( ".RAR" ) )
+         m_extensionList[ pos ] = ".rar";
+
+    if ( filelist.count() == 1)
+         target = filelist.first() + m_extensionList[ pos ];
+    else
     {
-        ++it;
-        target = item->url();
-        target.setPath( target.path( -1 ) + m_extensionList[ pos ] );
-        compressAs( item->url(), target );
+        target = m_dir + i18n("Archive") + m_extensionList[ pos ];
+        int i=1;
+        while ( KIO::NetAccess::exists(target, true, 0 ) )
+        {
+            target = m_dir + i18n("Archive %1").arg(i) + m_extensionList[ pos ];
+            i++;
+        }
     }
+    compressAs( filelist, target );
 
     extension = m_extensionList[ pos ];
     m_conf->setGroup( "ArkPlugin" );
@@ -419,104 +457,95 @@ void ArkMenu::slotCompressAs( int pos )
 
 void ArkMenu::slotCompressAsDefault()
 {
-    KFileItemListIterator it( m_list );
-    KFileItem * item;
     KURL name;
-    while (  ( item = it.current() ) != 0 )
+
+    if ( m_urlStringList.count() == 1)
+         name = m_urlStringList.first() + m_ext;
+    else
     {
-        ++it;
-        name = item->url();
-        name.setPath( name.path( -1 ) + m_ext );
-        compressAs( item->url(), name );
+        name = m_dir + i18n("Archive") + m_ext;
+        int i=1;
+        while ( KIO::NetAccess::exists(name, true, 0 ) )
+        {
+            name = m_dir + i18n("Archive %1").arg(i) + m_ext;
+            i++;
+        }
     }
+    compressAs( m_urlStringList, name );
 }
 
 // make work for URLs
-void ArkMenu::compressAs( const KURL & name, const KURL & compressed )
+void ArkMenu::compressAs( const QStringList &name, const KURL & compressed )
 {
     QStringList args;
-    args << "--add-to" << name.prettyURL() << compressed.prettyURL();
+    args << "--add-to";
+    args += name;
+    args << compressed.url();
     kapp->kdeinitExec( "ark", args );
 }
 
 void ArkMenu::slotAddTo( int pos )
 {
-    QStringList args;
-    args << "--add-to";
-    KFileItemListIterator it( m_list );
-    KFileItem * item;
-    while (  ( item = it.current() ) != 0 )
-    {
-        ++it;
-        args << item->url().prettyURL();
-    }
-    KURL archive( m_list.first()->url() );
+    QStringList args( m_urlStringList );
+    args.prepend( "--add-to" );
+
+    KURL archive( m_urlStringList.first() );
     archive.setPath( archive.directory( false ) );
     archive.setFileName( m_archiveList[ pos ].fileName() );
 
-    args << archive.prettyURL();
+    args << archive.url();
     kapp->kdeinitExec( "ark", args );
 }
 
 void ArkMenu::slotAdd()
 {
-    QStringList args;
-    args << "--add";
-    KFileItemListIterator it( m_list );
-    KFileItem * item;
-    while (  ( item = it.current() ) != 0 )
-    {
-        ++it;
-        args << item->url().prettyURL();
-    }
+    QStringList args( m_urlStringList );
+    args.prepend( "--add" );
+
     kapp->kdeinitExec( "ark", args );
 }
 
 void ArkMenu::slotExtractHere()
 {
-    QStringList args;
-    KFileItemListIterator it( m_list );
-    KFileItem * item;
-    while (  ( item = it.current() ) != 0 )
+    for ( QValueList<KURL>::ConstIterator it = m_urlList.constBegin();
+          it != m_urlList.constEnd();
+          ++it )
     {
-        args.clear();
-        ++it;
-        KURL targetDirectory = item->url();
+        QStringList args;
+        KURL targetDirectory = ( *it ).url();
         targetDirectory.setPath( targetDirectory.directory() );
-        args << "--extract-to"  << targetDirectory.prettyURL() << item->url().prettyURL();
+        args << "--extract-to"  << targetDirectory.url() << ( *it ).url();
         kapp->kdeinitExec( "ark", args );
     }
 }
 
 void ArkMenu::slotExtractToSubfolders()
 {
-    QStringList args;
-    QString dirName;
-    KURL targetDir;
-    KFileItemListIterator it( m_list );
-    KFileItem * item;
-    while (  ( item = it.current() ) != 0 )
+    for ( QStringList::ConstIterator it = m_urlStringList.constBegin();
+          it != m_urlStringList.constEnd();
+          ++it )
     {
-        args.clear();
-        ++it;
-        targetDir = item->url();
+        KURL targetDir;
+        QString dirName;
+        QStringList args;
+
+        targetDir = *it;
         dirName = targetDir.path();
         stripExtension( dirName );
         targetDir.setPath( dirName );
-        args << "--extract-to"  << targetDir.prettyURL() << item->url().prettyURL();
+        args << "--extract-to"  << targetDir.url() << *it;
         kapp->kdeinitExec( "ark", args );
     }
 }
 
 void ArkMenu::slotExtractTo()
 {
-    QStringList args;
-    KFileItemListIterator it( m_list );
-    KFileItem * item;
-    while (  ( item = it.current() ) != 0 )
+    for ( QStringList::ConstIterator it = m_urlStringList.constBegin();
+          it != m_urlStringList.constEnd();
+          ++it )
     {
-        ++it;
-        args << "--extract"  << item->url().prettyURL();
+        QStringList args;
+        args << "--extract" << *it;
         kapp->kdeinitExec( "ark", args );
     }
 }
